@@ -1,7 +1,5 @@
 # coding: utf-8
 
-from __future__ import absolute_import, print_function
-
 """
 stuff to deal with comments and formatting on dict/list/ordereddict/set
 these are not really related, formatting could be factored out as
@@ -13,14 +11,11 @@ import copy
 
 
 from ruamel.yaml.compat import ordereddict  # type: ignore
-from ruamel.yaml.compat import PY2, string_types, MutableSliceableSequence
+from ruamel.yaml.compat import MutableSliceableSequence, _F, nprintf  # NOQA
 from ruamel.yaml.scalarstring import ScalarString
 from ruamel.yaml.anchor import Anchor
 
-if PY2:
-    from collections import MutableSet, Sized, Set, Mapping
-else:
-    from collections.abc import MutableSet, Sized, Set, Mapping
+from collections.abc import MutableSet, Sized, Set, Mapping
 
 if False:  # MYPY
     from typing import Any, Dict, Optional, List, Union, Optional, Iterator  # NOQA
@@ -28,9 +23,62 @@ if False:  # MYPY
 # fmt: off
 __all__ = ['CommentedSeq', 'CommentedKeySeq',
            'CommentedMap', 'CommentedOrderedMap',
-           'CommentedSet', 'comment_attrib', 'merge_attrib']
+           'CommentedSet', 'comment_attrib', 'merge_attrib',
+           'C_POST', 'C_PRE', 'C_SPLIT_ON_FIRST_BLANK', 'C_BLANK_LINE_PRESERVE_SPACE',
+           ]
 # fmt: on
 
+# splitting of comments by the scanner
+# an EOLC (End-Of-Line Comment) is preceded by some token
+# an FLC (Full Line Comment) is a comment not preceded by a token, i.e. # is
+#   the first non-blank on line
+# a BL is a blank line i.e. empty or spaces/tabs only
+# bits 0 and 1 are combined, you can choose only one
+C_POST = 0b00
+C_PRE = 0b01
+C_SPLIT_ON_FIRST_BLANK = 0b10  # as C_POST, but if blank line then C_PRE all lines before
+# first blank goes to POST even if no following real FLC
+# (first blank -> first of post)
+# 0b11 -> reserved for future use
+C_BLANK_LINE_PRESERVE_SPACE = 0b100
+# C_EOL_PRESERVE_SPACE2 = 0b1000
+
+
+class IDX:
+    # temporary auto increment, so rearranging is easier
+    def __init__(self):
+        # type: () -> None
+        self._idx = 0
+
+    def __call__(self):
+        # type: () -> Any
+        x = self._idx
+        self._idx += 1
+        return x
+
+    def __str__(self):
+        # type: () -> Any
+        return str(self._idx)
+
+
+cidx = IDX()
+
+# more or less in order of subjective expected likelyhood
+# the _POST and _PRE ones are lists themselves
+C_VALUE_EOL = C_ELEM_EOL = cidx()
+C_KEY_EOL = cidx()
+C_KEY_PRE = C_ELEM_PRE = cidx()  # not this is not value
+C_VALUE_POST = C_ELEM_POST = cidx()  # not this is not value
+C_VALUE_PRE = cidx()
+C_KEY_POST = cidx()
+C_TAG_EOL = cidx()
+C_TAG_POST = cidx()
+C_TAG_PRE = cidx()
+C_ANCHOR_EOL = cidx()
+C_ANCHOR_POST = cidx()
+C_ANCHOR_PRE = cidx()
+
+
 comment_attrib = '_yaml_comment'
 format_attrib = '_yaml_format'
 line_col_attrib = '_yaml_line_col'
@@ -38,30 +86,67 @@ merge_attrib = '_yaml_merge'
 tag_attrib = '_yaml_tag'
 
 
-class Comment(object):
-    # sys.getsize tested the Comment objects, __slots__ makes them bigger
+class Comment:
+    # using sys.getsize tested the Comment objects, __slots__ makes them bigger
     # and adding self.end did not matter
-    __slots__ = 'comment', '_items', '_end', '_start'
+    __slots__ = 'comment', '_items', '_post', '_pre'
     attrib = comment_attrib
 
-    def __init__(self):
-        # type: () -> None
+    def __init__(self, old=True):
+        # type: (bool) -> None
+        self._pre = None if old else []  # type: ignore
         self.comment = None  # [post, [pre]]
         # map key (mapping/omap/dict) or index (sequence/list) to a  list of
         # dict: post_key, pre_key, post_value, pre_value
         # list: pre item, post item
         self._items = {}  # type: Dict[Any, Any]
         # self._start = [] # should not put these on first item
-        self._end = []  # type: List[Any] # end of document comments
+        self._post = []  # type: List[Any] # end of document comments
 
     def __str__(self):
         # type: () -> str
-        if bool(self._end):
-            end = ',\n  end=' + str(self._end)
+        if bool(self._post):
+            end = ',\n  end=' + str(self._post)
         else:
             end = ""
         return 'Comment(comment={0},\n  items={1}{2})'.format(self.comment, self._items, end)
 
+    def _old__repr__(self):
+        # type: () -> str
+        if bool(self._post):
+            end = ',\n  end=' + str(self._post)
+        else:
+            end = ""
+        try:
+            ln = max([len(str(k)) for k in self._items]) + 1  # type: ignore
+        except ValueError:
+            ln = ''  # type: ignore
+        it = '    '.join(
+            ['{:{}} {}\n'.format(str(k) + ':', ln, v) for k, v in self._items.items()]
+        )
+        if it:
+            it = '\n    ' + it + '  '
+        return 'Comment(\n  start={},\n  items={{{}}}{})'.format(self.comment, it, end)
+
+    def __repr__(self):
+        # type: () -> str
+        if self._pre is None:
+            return self._old__repr__()
+        if bool(self._post):
+            end = ',\n  end=' + repr(self._post)
+        else:
+            end = ""
+        try:
+            ln = max([len(str(k)) for k in self._items]) + 1  # type: ignore
+        except ValueError:
+            ln = ''  # type: ignore
+        it = '    '.join(
+            ['{:{}} {}\n'.format(str(k) + ':', ln, v) for k, v in self._items.items()]
+        )
+        if it:
+            it = '\n    ' + it + '  '
+        return 'Comment(\n  pre={},\n  items={{{}}}{})'.format(self.pre, it, end)
+
     @property
     def items(self):
         # type: () -> Any
@@ -70,22 +155,62 @@ class Comment(object):
     @property
     def end(self):
         # type: () -> Any
-        return self._end
+        return self._post
 
     @end.setter
     def end(self, value):
         # type: (Any) -> None
-        self._end = value
+        self._post = value
 
     @property
-    def start(self):
+    def pre(self):
         # type: () -> Any
-        return self._start
+        return self._pre
 
-    @start.setter
-    def start(self, value):
+    @pre.setter
+    def pre(self, value):
         # type: (Any) -> None
-        self._start = value
+        self._pre = value
+
+    def get(self, item, pos):
+        # type: (Any, Any) -> Any
+        x = self._items.get(item)
+        if x is None or len(x) < pos:
+            return None
+        return x[pos]  # can be None
+
+    def set(self, item, pos, value):
+        # type: (Any, Any, Any) -> Any
+        x = self._items.get(item)
+        if x is None:
+            self._items[item] = x = [None] * (pos + 1)
+        else:
+            while len(x) <= pos:
+                x.append(None)
+        assert x[pos] is None
+        x[pos] = value
+
+    def __contains__(self, x):
+        # type: (Any) -> Any
+        # test if a substring is in any of the attached comments
+        if self.comment:
+            if self.comment[0] and x in self.comment[0].value:
+                return True
+            if self.comment[1]:
+                for c in self.comment[1]:
+                    if x in c.value:
+                        return True
+        for value in self.items.values():
+            if not value:
+                continue
+            for c in value:
+                if c and x in c.value:
+                    return True
+        if self.end:
+            for c in self.end:
+                if x in c.value:
+                    return True
+        return False
 
 
 # to distinguish key from None
@@ -94,7 +219,7 @@ def NoComment():
     pass
 
 
-class Format(object):
+class Format:
     __slots__ = ('_flow_style',)
     attrib = format_attrib
 
@@ -121,7 +246,11 @@ class Format(object):
         return self._flow_style
 
 
-class LineCol(object):
+class LineCol:
+    """
+    line and column information wrt document, values start at zero (0)
+    """
+
     attrib = line_col_attrib
 
     def __init__(self):
@@ -163,8 +292,12 @@ class LineCol(object):
             self.data = {}
         self.data[key] = data
 
+    def __repr__(self):
+        # type: () -> str
+        return _F('LineCol({line}, {col})', line=self.line, col=self.col)  # type: ignore
 
-class Tag(object):
+
+class Tag:
     """store tag information for roundtripping"""
 
     __slots__ = ('value',)
@@ -179,7 +312,7 @@ class Tag(object):
         return '{0.__class__.__name__}({0.value!r})'.format(self)
 
 
-class CommentedBase(object):
+class CommentedBase:
     @property
     def ca(self):
         # type: () -> Any
@@ -225,7 +358,7 @@ class CommentedBase(object):
         from .error import CommentMark
         from .tokens import CommentToken
 
-        pre_comments = self._yaml_get_pre_comment()
+        pre_comments = self._yaml_clear_pre_comment()  # type: ignore
         if comment[-1] == '\n':
             comment = comment[:-1]  # strip final newline if there
         start_mark = CommentMark(indent)
@@ -233,7 +366,7 @@ class CommentedBase(object):
             c = com.strip()
             if len(c) > 0 and c[0] != '#':
                 com = '# ' + com
-            pre_comments.append(CommentToken(com + '\n', start_mark, None))
+            pre_comments.append(CommentToken(com + '\n', start_mark))
 
     def yaml_set_comment_before_after_key(
         self, key, before=None, indent=0, after=None, after_indent=None
@@ -248,7 +381,7 @@ class CommentedBase(object):
         def comment_token(s, mark):
             # type: (Any, Any) -> Any
             # handle empty lines as having no comment
-            return CommentToken(('# ' if s else "") + s + '\n', mark, None)
+            return CommentToken(('# ' if s else "") + s + '\n', mark)
 
         if after_indent is None:
             after_indent = indent + 2
@@ -302,7 +435,7 @@ class CommentedBase(object):
                 comment = ' ' + comment
                 column = 0
         start_mark = CommentMark(column)
-        ct = [CommentToken(comment, start_mark, None), None]
+        ct = [CommentToken(comment, start_mark), None]
         self._yaml_add_eol_comment(ct, key=key)
 
     @property
@@ -395,7 +528,7 @@ class CommentedSeq(MutableSliceableSequence, list, CommentedBase):  # type: igno
         # try to preserve the scalarstring type if setting an existing key to a new value
         if idx < len(self):
             if (
-                isinstance(value, string_types)
+                isinstance(value, str)
                 and not isinstance(value, ScalarString)
                 and isinstance(self[idx], ScalarString)
             ):
@@ -469,6 +602,15 @@ class CommentedSeq(MutableSliceableSequence, list, CommentedBase):  # type: igno
         return column
 
     def _yaml_get_pre_comment(self):
+        # type: () -> Any
+        pre_comments = []  # type: List[Any]
+        if self.ca.comment is None:
+            self.ca.comment = [None, pre_comments]
+        else:
+            pre_comments = self.ca.comment[1]
+        return pre_comments
+
+    def _yaml_clear_pre_comment(self):
         # type: () -> Any
         pre_comments = []  # type: List[Any]
         if self.ca.comment is None:
@@ -552,6 +694,15 @@ class CommentedKeySeq(tuple, CommentedBase):  # type: ignore
         return column
 
     def _yaml_get_pre_comment(self):
+        # type: () -> Any
+        pre_comments = []  # type: List[Any]
+        if self.ca.comment is None:
+            self.ca.comment = [None, pre_comments]
+        else:
+            pre_comments = self.ca.comment[1]
+        return pre_comments
+
+    def _yaml_clear_pre_comment(self):
         # type: () -> Any
         pre_comments = []  # type: List[Any]
         if self.ca.comment is None:
@@ -691,6 +842,15 @@ class CommentedMap(ordereddict, CommentedBase):  # type: ignore
         return column
 
     def _yaml_get_pre_comment(self):
+        # type: () -> Any
+        pre_comments = []  # type: List[Any]
+        if self.ca.comment is None:
+            self.ca.comment = [None, pre_comments]
+        else:
+            pre_comments = self.ca.comment[1]
+        return pre_comments
+
+    def _yaml_clear_pre_comment(self):
         # type: () -> Any
         pre_comments = []  # type: List[Any]
         if self.ca.comment is None:
@@ -767,7 +927,7 @@ class CommentedMap(ordereddict, CommentedBase):  # type: ignore
         # try to preserve the scalarstring type if setting an existing key to a new value
         if key in self:
             if (
-                isinstance(value, string_types)
+                isinstance(value, str)
                 and not isinstance(value, ScalarString)
                 and isinstance(self[key], ScalarString)
             ):
@@ -840,75 +1000,22 @@ class CommentedMap(ordereddict, CommentedBase):  # type: ignore
         # type: (Any) -> bool
         return bool(dict(self) == other)
 
-    if PY2:
+    def keys(self):
+        # type: () -> Any
+        return CommentedMapKeysView(self)
 
-        def keys(self):
-            # type: () -> Any
-            return list(self._keys())
-
-        def iterkeys(self):
-            # type: () -> Any
-            return self._keys()
-
-        def viewkeys(self):
-            # type: () -> Any
-            return CommentedMapKeysView(self)
-
-    else:
-
-        def keys(self):
-            # type: () -> Any
-            return CommentedMapKeysView(self)
-
-    if PY2:
-
-        def _values(self):
-            # type: () -> Any
-            for x in ordereddict.__iter__(self):
-                yield ordereddict.__getitem__(self, x)
-
-        def values(self):
-            # type: () -> Any
-            return list(self._values())
-
-        def itervalues(self):
-            # type: () -> Any
-            return self._values()
-
-        def viewvalues(self):
-            # type: () -> Any
-            return CommentedMapValuesView(self)
-
-    else:
-
-        def values(self):
-            # type: () -> Any
-            return CommentedMapValuesView(self)
+    def values(self):
+        # type: () -> Any
+        return CommentedMapValuesView(self)
 
     def _items(self):
         # type: () -> Any
         for x in ordereddict.__iter__(self):
             yield x, ordereddict.__getitem__(self, x)
 
-    if PY2:
-
-        def items(self):
-            # type: () -> Any
-            return list(self._items())
-
-        def iteritems(self):
-            # type: () -> Any
-            return self._items()
-
-        def viewitems(self):
-            # type: () -> Any
-            return CommentedMapItemsView(self)
-
-    else:
-
-        def items(self):
-            # type: () -> Any
-            return CommentedMapItemsView(self)
+    def items(self):
+        # type: () -> Any
+        return CommentedMapItemsView(self)
 
     @property
     def merge(self):
@@ -978,10 +1085,7 @@ class CommentedKeyMap(CommentedBase, Mapping):  # type: ignore
         try:
             self._od = ordereddict(*args, **kw)
         except TypeError:
-            if PY2:
-                self._od = ordereddict(args[0].items())
-            else:
-                raise
+            raise
 
     __delitem__ = __setitem__ = clear = pop = popitem = setdefault = update = raise_immutable
 
@@ -1140,14 +1244,14 @@ def dump_comments(d, name="", sep='.', out=sys.stdout):
     """
     if isinstance(d, dict) and hasattr(d, 'ca'):
         if name:
-            sys.stdout.write('{}\n'.format(name))
-        out.write('{}\n'.format(d.ca))  # type: ignore
+            out.write('{} {}\n'.format(name, type(d)))
+        out.write('{!r}\n'.format(d.ca))  # type: ignore
         for k in d:
-            dump_comments(d[k], name=(name + sep + k) if name else k, sep=sep, out=out)
+            dump_comments(d[k], name=(name + sep + str(k)) if name else k, sep=sep, out=out)
     elif isinstance(d, list) and hasattr(d, 'ca'):
         if name:
-            sys.stdout.write('{}\n'.format(name))
-        out.write('{}\n'.format(d.ca))  # type: ignore
+            out.write('{} {}\n'.format(name, type(d)))
+        out.write('{!r}\n'.format(d.ca))  # type: ignore
         for idx, k in enumerate(d):
             dump_comments(
                 k, name=(name + sep + str(idx)) if name else str(idx), sep=sep, out=out
